@@ -157,7 +157,7 @@ function(global.model, beta = FALSE, evaluate = TRUE, rank = "AICc",
 	nov <- as.integer(n.vars - n.fixed)
 	ncomb <- (2L ^ nov) * nvariants
 
-	if(nov > 31L) stop(gettextf("number of predictors (%d) exceeds allowed maximum (31)"), nov, domain = "MuMIn")
+	if(nov > 31L) stop(gettextf("number of predictors (%d) exceeds allowed maximum (31)", nov, domain = "MuMIn"))
 	#if(nov > 10L) warning(gettextf("%d predictors will generate up to %.0f combinations", nov, ncomb))
 	nmax <- ncomb * nvariants
 	if(evaluate) {
